<?php
/**
 * @file
 * Enables modules and site configuration for a opencrm site installation.
 */

use Drupal\user\Entity\User;
use Drupal\Core\Form\FormStateInterface;

/**
 * Implements hook_install_tasks().
 */
function opencrm_kickstart_install_tasks(&$install_state) {
  $tasks = array(
    'opencrm_kickstart_install_profile_modules' => array(
      'display_name' => t('Install OpenCRM Kickstart modules'),
      'type' => 'batch',
    ),
    'opencrm_kickstart_final_site_setup' => array(
      'display_name' => t('Apply configuration'),
      'type' => 'batch',
      'display' => TRUE,
    ),
  );
  return $tasks;
}

/**
 * Implements hook_install_tasks_alter().
 */
function opencrm_kickstart_install_tasks_alter(&$tasks, $install_state) {
  // Override the core finished task function.
  $tasks['install_finished']['function'] = 'opencrm_kickstart_install_finished';
}

/**
 * Implements hook_form_FORM_ID_alter() for install_configure_form().
 *
 * Allows the profile to alter the site configuration form.
 */
function opencrm_kickstart_form_install_configure_form_alter(&$form, FormStateInterface $form_state) {
  $form['opencrm_kickstart'] = [
    '#type' => 'fieldgroup',
    '#title' => t('OpenCRM Kickstart optional configuration'),
    '#description' => t('All the required modules and configuration will be automatically installed and imported. You can optionally generated demo content.'),
    '#weight' => 50,
  ];

  // Checkboxes to generate demo content.
  $form['opencrm_kickstart']['demo_content'] = [
    '#type' => 'checkbox',
    '#title' => t('Generate demo crm content and users'),
  ];

  // Submit handler to generate demo content.
  $form['#submit'][] = 'opencrm_kickstart_demo_submit';
}

/**
 * Submit handler.
 */
function opencrm_kickstart_demo_submit($form_id, &$form_state) {
  \Drupal::state()->set('opencrm_kickstart_install_demo_content', $form_state->getValue('demo_content'));
}

/**
 * Installs required modules via a batch process.
 *
 * @param $install_state
 *   An array of information about the current installation state.
 *
 * @return array
 *   The batch definition.
 */
function opencrm_kickstart_install_profile_modules(&$install_state) {
  $files = system_rebuild_module_data();

  $modules = array(
    'toolbar' => 'toolbar',
    'contacts' => 'contacts',
  );
  $opencrm_modules = $modules;
  // Always install required modules first. Respect the dependencies between
  // the modules.
  $required = array();
  $non_required = array();

  // Add modules that other modules depend on.
  foreach ($modules as $module) {
    if ($files[$module]->requires) {
      $module_requires = array_keys($files[$module]->requires);
      // Remove the opencrm_kickstart modules from required modules.
      $module_requires = array_diff_key($module_requires, $opencrm_modules);
      $modules = array_merge($modules, $module_requires);
    }
  }
  $modules = array_unique($modules);
  // Remove the opencrm_kickstart modules from to install modules.
  $modules = array_diff_key($modules, $opencrm_modules);
  foreach ($modules as $module) {
    if (!empty($files[$module]->info['required'])) {
      $required[$module] = $files[$module]->sort;
    }
    else {
      $non_required[$module] = $files[$module]->sort;
    }
  }
  arsort($required);

  $operations = array();
  foreach ($required + $non_required + $opencrm_modules as $module => $weight) {
    $operations[] = array('_opencrm_kickstart_install_module_batch', array(array($module), $module));
  }

  $batch = array(
    'operations' => $operations,
    'title' => t('Install OpenCRM Kickstart modules'),
    'error_message' => t('The installation has encountered an error.'),
  );
  return $batch;
}

/**
 * Set up final task to complete installation.
 */
function opencrm_kickstart_final_site_setup(&$install_state) {
  // Clear all status messages generated by modules installed in previous step.
  drupal_get_messages('status', TRUE);

  $batch = array();

  $demo_content = \Drupal::state()->get('opencrm_kickstart_install_demo_content');
  if ($demo_content === 1) {
    // Install demo content modules.
    $batch['operations'][] = ['_opencrm_kickstart_install_module_batch', array(array('default_content'), 'default_content')];
    $batch['operations'][] = ['_opencrm_kickstart_install_module_batch', array(array('opencrm_demo'), 'opencrm_demo')];

    // Uninstall demo content modules.
    $batch['operations'][] = ['_opencrm_kickstart_uninstall_module_batch', array(array('default_content'), 'default_content')];
    $batch['operations'][] = ['_opencrm_kickstart_uninstall_module_batch', array(array('opencrm_demo'), 'opencrm_demo')];
  }

  // Add some finalising steps.
  $final_batched = [
    'profile_weight' => t('Set weight of profile.'),
    'router_rebuild' => t('Rebuild router.'),
    'cron_run' => t('Run cron.'),
  ];
  foreach ($final_batched as $process => $description) {
    $batch['operations'][] = ['_opencrm_kickstart_finalise_batch', array($process, $description)];
  }

  return $batch;
}

/**
 * Performs final installation steps.
 *
 * @param $install_state
 *   An array of information about the current installation state.
 *
 * @see install_finished().
 */
function opencrm_kickstart_install_finished(&$install_state) {
  // Clear all status messages generated by modules installed in previous step.
  drupal_get_messages('status', TRUE);

  if ($install_state['interactive']) {
    // Load current user and perform final login tasks.
    // This has to be done after drupal_flush_all_caches()
    // to avoid session regeneration.
    $account = User::load(1);
    user_login_finalize($account);
  }
}

/**
 * Implements callback_batch_operation().
 *
 * Performs batch installation of modules.
 */
function _opencrm_kickstart_install_module_batch($module, $module_name, &$context) {
  set_time_limit(0);
  \Drupal::service('module_installer')->install($module, $dependencies = TRUE);
  $context['results'][] = $module;
  $context['message'] = t('Install %module_name module.', array('%module_name' => $module_name));
}

/**
 * Implements callback_batch_operation().
 *
 * Performs batch uninstallation of modules.
 */
function _opencrm_kickstart_uninstall_module_batch($module, $module_name, &$context) {
  set_time_limit(0);
  \Drupal::service('module_installer')->uninstall($module, $dependencies = FALSE);
  $context['results'][] = $module;
  $context['message'] = t('Uninstalled %module_name module.', array('%module_name' => $module_name));
}

/**
 * Implements callback_batch_operation().
 *
 * Performs batch finalising.
 */
function _opencrm_kickstart_finalise_batch($process, $description, &$context) {
  switch ($process) {
    case 'profile_weight':
      $profile = drupal_get_profile();

      // Installation profiles are always loaded last.
      module_set_weight($profile, 1000);
      break;

    case 'router_rebuild':
      // Build the router once after installing all modules.
      // This would normally happen upon KernelEvents::TERMINATE, but since the
      // installer does not use an HttpKernel, that event is never triggered.
      \Drupal::service('router.builder')->rebuild();
      break;

    case 'cron_run':
      // Run cron to populate update status tables (if available) so that users
      // will be warned if they've installed an out of date Drupal version.
      // Will also trigger indexing of profile-supplied content or feeds.
      \Drupal::service('cron')->run();
      break;

  }

  $context['results'][] = $process;
  $context['message'] = $description;
}
